#!/bin/bash
#
# IceWarp Server installation package
# Copyright (c) 2008-2010 IceWarp Ltd. All rights reserved.
#
# http://www.icewarp.com 
#
# file: install.sh - install script
# version: 1.5
#

# variables
EMPTY_VALUE="UNKNOWN"

USE_COLORS="0"

IMAGENAME="IceWarpServer-image-10.1.4_20101015_RHEL5.tar"

SETUP_NEWVERSION="10.1.4 (2010-10-15) RHEL5"
SETUP_OLDVERSION="$EMPTY_VALUE"

SETUP_INSTALL_DIR="$EMPTY_VALUE"
SETUP_INSTALL_DIR_DEFAULT="/opt/icewarp"

SETUP_INSTALL_USER="$EMPTY_VALUE"
SETUP_INSTALL_USER_DEFAULT=$(id -un)

SETUP_INSTALL_SERVICE_NAME="icewarp"

SETUP_SERVER_CONF="$EMPTY_VALUE"
SETUP_SERVER_CONF_EXISTING="$EMPTY_VALUE"

SETUP_CONFIG_FILE_EXISTS="0"

INSTALL_LOG=~/icewarp-install.log
INSTALL_ERROR_LOG=~/icewarp-install-error.log

RUNNING_UID=$(id -u)

UPGRADE="0"

OS_DIST="$EMPTY_VALUE"

SETTINGS_DIR_CALENDAR="$EMPTY_VALUE"
SETTINGS_DIR_CONFIG="$EMPTY_VALUE"
SETTINGS_DIR_SPAM="$EMPTY_VALUE"

# init

SELF=$0
WORKDIR=$(dirname $0)

if  [ "$TERM" = "linux" ] || [ "$TERM" = "xterm" ] || [ "$TERM" = "xterm-color" ] || [ "$TERM" = "screen" ] && [ "$USE_COLORS" = "1" ]; then
    GOOD=$'\e[32;01m'
    WARN=$'\e[33;01m'
    BAD=$'\e[31;01m'
    NORMAL=$'\e[0m'
    HILITE=$'\e[37;01m'
    BRACKET=$'\e[34;01m'
else
    GOOD=$'\e[0;32m'
    BAD=$'\e[0;31m'
    NORMAL=$'\e[0m'
    WARN=${NORMAL}
    HILITE=${NORMAL}
    BRACKET=${NORMAL}
fi

good()
{
    echo -e "${GOOD}**${NORMAL}\t$*"
}

bad()
{
    echo -e "${BAD}**${NORMAL}\tError: $*"
}

warn()
{
    echo -e "${WARN}**${NORMAL}\tWarning: $*"
}

hilite()
{
    echo -e "${GOOD}**${HILITE}\t$*${NORMAL}"
}

getparam()
{
    echo -e -n "${GOOD}**${NORMAL}\t$1 [$2]${NORMAL}: "
    read PARAM
}

# Detection of host OS distribution
# Function tryes to detect OS in simple way
# Result of detection is:
# - "UNKNOWN" if distributionplatform is not known
# - "RHEL" if OS is Red Hat Enterprise Linux
detect_os_distribution()
{
    OS_DIST="UNKNOWN"

    # TODO: DISTRIBUTOR_ID=$(lsb_release -i)
    if [ -f "/etc/redhat-release" ]; then
        OS_DIST="RHEL"
    fi
}

# Ask question with confirmation
# Parameters:
# $1 - question
# $2 - first choice, it is choosed if user enters only empty string
# $3 - second choice
# Return:
# 0 on error - unknown string entered
# 1 if first choice was selected
# 2 if second choice was selected
ask_with_confirmation()
{
    if [ -z "$1" ] || [ -z "$2" ] || [ -z "$3" ]; then
        return 0
    fi
    LOWER_CHOICE_1=$(echo "$2" | tr "A-Z" "a-z")
    LOWER_CHOICE_2=$(echo "$3" | tr "A-Z" "a-z")
    echo -e -n "${GOOD}**${NORMAL}\t${1} [${HILITE}${2}${NORMAL}/${3}]: "
    read PARAM
    PARAM=$(echo "$PARAM" | tr "A-Z" "a-z")
    if [ "$PARAM" == "" ] || [ "$PARAM" == "$LOWER_CHOICE_1" ]; then
        return 1
    fi
    if [ "$PARAM" == "$LOWER_CHOICE_2" ]; then
        return 2
    fi
    return 0
}

preconf()
{
    # reset variables for old configuration
    InstallDir=""
    User=""
    Version=""

    if [ -f "$1" ]; then
        source "$1"

        # check for new variables first

        # check for install dir
        if [ "x${IWS_INSTALL_DIR}" != "x" ]; then
            SETUP_INSTALL_DIR="$IWS_INSTALL_DIR"
        fi

        # check for install user
        if [ "x${IWS_PROCESS_USER}" != "x" ]; then
            SETUP_INSTALL_USER="$IWS_PROCESS_USER"
        fi

        # check for installed version
        if [ "x${IWS_VERSION}" != "x" ]; then
            SETUP_OLDVERSION="$IWS_VERSION"
        fi

        # check for old variables

        # check for install dir
        if [ "x${InstallDir}" != "x" ]; then
            SETUP_INSTALL_DIR="$InstallDir"
        fi

        if [ "x${User}" != "x" ]; then
            SETUP_INSTALL_USER="$User"
        fi

        if [ "x${Version}" != "x" ]; then
            SETUP_OLDVERSION="$Version"
        fi
    fi
}

# Function displays license file and waits until user press [ENTER]
accept_license()
{
    less "${WORKDIR}/LICENSE"
    echo ""
    good "You must accept this license agreement if you want to continue."
    good "Press ENTER to accept license or CTRL+C to quit"
    read -s
    echo ""
}

# Function diplays text about running server and waits until user press [ENTER]
check_server_is_running()
{
    good "Please make sure the IceWarp Server is not running before installing."
    good "Press ENTER to continue, CTRL+C to quit"
    read -s
    echo ""
}

# Function checks if configuration file exists and sets variables
# Parameters
#  $1 - full path to assumed configuration file
# Returns:
#  0 - if all variables SETUP_INSTALL_DIR, SETUP_INSTALL_USER are detected
# != 0 - if some of variables or non of variables SETUP_INSTALL_DIR, SETUP_INSTALL_USER are detected
detect_install_options_sub()
{
    CONFIG_FILE="$1"

    # for safety initialize variables
    SETUP_INSTALL_DIR="$EMPTY_VALUE"
    SETUP_INSTALL_USER="$EMPTY_VALUE"
    SETUP_OLDVERSION="$EMPTY_VALUE"

    if [ -f "$CONFIG_FILE" ]; then
        preconf "$CONFIG_FILE"
        # check for set values
        if [ "x${SETUP_INSTALL_DIR}" != "x${EMPTY_VALUE}" ] &&
           [ "x${SETUP_INSTALL_USER}" != "x${EMPTY_VALUE}" ] &&
           [ "x${SETUP_OLDVERSION}" != "x${EMPTY_VALUE}" ]; then
           # all values was set
           return 0
        fi
    fi

    # configuration file not found, or some values was not set
    # reset variables
    SETUP_INSTALL_DIR="$EMPTY_VALUE"
    SETUP_INSTALL_USER="$EMPTY_VALUE"
    SETUP_OLDVERSION="$EMPTY_VALUE"
    return 1
}

# Function detects install options entered by user from already installed server
# If variables are not detected, then default values are set
# Load balanced setup is not detected here, because for detecting load balanced setup tool needs to be called,
# so server configuration file must be created first.
# If configuration file is found, flag SETUP_CONFIG_FILE_EXISTS is set to value other than 0
# Variables detected:
# 1. install dir default - SETUP_INSTALL_DIR_DEFAULT
# 2. install user - SETUP_INSTALL_USER
# returns:
# always 0, if install options are not detected they are set to default values
detect_install_options()
{
    # detect configuration files from newest to oldest,
    # Merak 9.4 upgraded to IceWarp Server 10 can be already installed or
    # clean IceWarp Server 10 can be already installed

    # assume configuration file exists
    SETUP_CONFIG_FILE_EXISTS="1"

    detect_install_options_sub "/etc/icewarp/icewarp.conf"
    if [ $? -eq 0 ]; then
        return 0
    fi

    # directory /opt/icewarp

    # IceWarp Server 10
    detect_install_options_sub "/opt/icewarp/config/icewarp.conf"
    if [ $? -eq 0 ]; then
        return 0
    fi

    # Merak 9.4
    detect_install_options_sub "/opt/icewarp/config/merak.conf"
    if [ $? -eq 0 ]; then
        return 0
    fi

    # directory /opt/merak
    
    # IceWarp Server 10
    detect_install_options_sub "/opt/merak/config/icewarp.conf"
    if [ $? -eq 0 ]; then
        return 0
    fi

    # Merak 9.4
    detect_install_options_sub "/opt/merak/config/merak.conf"
    if [ $? -eq 0 ]; then
        return 0
    fi

    # set default values
    SETUP_INSTALL_DIR="$SETUP_INSTALL_DIR_DEFAULT"
    SETUP_INSTALL_USER="$SETUP_INSTALL_USER_DEFAULT"

    # configuration file not found
    SETUP_CONFIG_FILE_EXISTS="0"

    return 1
}

# Function displays questions about SETUP_INSTALL_DIR, SETUP_INSTALL_USER
# Function detect_install_options() must be called first
# returns:
#  no return
ask_questions()
{
    getparam "Installation prefix" "$SETUP_INSTALL_DIR"
    if [ "x${PARAM}" != "x" ]; then
        SETUP_INSTALL_DIR="$PARAM"
    fi

    # check for "new installation" or "upgrade"
    # check if installation directory exist
    if [ -d "$SETUP_INSTALL_DIR" ]; then
        UPGRADE="2"
    fi
    # check if installation file exist but it is not directory
    if [ "$UPGRADE" == "0" -a -e "$SETUP_INSTALL_DIR" ]; then
        bad "File ${SETUP_INSTALL_DIR} exists but it is not directory."
        exit 1
    fi

    if [ "$SETUP_CONFIG_FILE_EXISTS" != "0" ]; then
        UPGRADE="1"
    fi

    # if UPGRADE is 2 then SETUP_INSTALL_DIR exists but configuration file does not exist
    if [ "$UPGRADE" == "2" ]; then
        good ""
        good "Directory ${SETUP_INSTALL_DIR} exists but"
        good "server configuration file does not exist."
        good ""
        good "Please select full installation or upgrade."
        good "WARNING: directory ${SETUP_INSTALL_DIR} will be DELETED in full installation!"
        good ""
        ask_with_confirmation "Upgrade or Install?" "U" "i"
        if [ $? -eq 1 ]; then
            UPGRADE="1"
        else
            UPGRADE="0"
        fi
    fi

    if [ "$UPGRADE" == "1" ]; then
        echo "${GOOD}** Performing upgrade"
    else
        echo "${GOOD}** Performing new install"

        getparam "Run services as user" "$SETUP_INSTALL_USER"
        if [ "x$PARAM" != "x" ]; then
            SETUP_INSTALL_USER="$PARAM"
        fi
    fi

    # set server configuration file
    SETUP_SERVER_CONF="/etc/icewarp/icewarp.conf"
}

# Function checks for entered install options. Failed check causes script exit.
# Parameters:
#  none
# Returns:
#  none
check_install_options()
{
    # test if user exists
    id "$SETUP_INSTALL_USER" &> /dev/null
    if [ $? -ne 0 ]; then
        bad "User ${SETUP_INSTALL_USER} doesn't exist."
        exit 1
    fi

    # test if user is in group
    id -g "$SETUP_INSTALL_USER" &> /dev/null
    if [ $? -ne 0 ]; then
        bad "User ${SETUP_INSTALL_USER} is not in group."
        exit 1
    fi
}

# Function asks user to confirm entered installation options
# Parameters:
#  none
# Returns:
#  none
confirm_install_options()
{
    SETUP_INSTALL_GROUP=$(id -gn "$SETUP_INSTALL_USER")

    good ""
    good "Please check entered informations before continuing:"
    good ""
    if [ "$UPGRADE" == "1" ]; then
        good "Installation prefix:\t\t${SETUP_INSTALL_DIR} (upgrading)"
    else
        good "Installation prefix:\t\t${SETUP_INSTALL_DIR} (directory will be created)"
    fi

    good "IceWarp Server will run as user:\t${SETUP_INSTALL_USER}"
    good "IceWarp Server will run as group:\t${SETUP_INSTALL_GROUP}"

    good ""
    good "Press ENTER to continue, CTRL+C to quit"
    read -s
    echo ""
}

# Function creates destination directory, extracts tar image
# and in case of installing, it copies default configuration
# files, so all not-generated files are on the place
# Parameters:
#   None
# Returns:
#   None
extract_package()
{
    if [ "$UPGRADE" == "0" ]; then
        # Install

        good "Creating ${SETUP_INSTALL_DIR} directory ..."

        if [ "x${SETUP_INSTALL_DIR}" == "x/" ]; then
	        bad "Rejecting directory removal: ${SETUP_INSTALL_DIR}"
            exit 1
        fi
        rm -rf "$SETUP_INSTALL_DIR"
        mkdir -p "$SETUP_INSTALL_DIR"
        if [ $? -ne 0 ]; then
            bad "Failed! Check user permissions"
            exit 1
        fi
    else
        # Upgrade

        # remove files which are not in use anymore
        # but not changed by extracting from tar

        # files are removed in final_cleanup()
        true
    fi
    good "Extracting data ..."
    tar -xf "${WORKDIR}/${IMAGENAME}" -C "$SETUP_INSTALL_DIR"
    if [ $? -ne 0 ]; then
        bad "Failed, check user permissions and available disk space. 256 MB disk space is needed."
        exit 1
    fi

	# In case of installing, copy default configuration files
    if [ "$UPGRADE" == "0" ]; then
        # copy all configuration files

        # calendar
        cp -f "${SETUP_INSTALL_DIR}/calendar/default/calendar.dat" "${SETUP_INSTALL_DIR}/calendar/calendar.dat"
        cp -f "${SETUP_INSTALL_DIR}/calendar/default/db/groupware.db" "${SETUP_INSTALL_DIR}/calendar/groupware.db"

        # config
        cp -f "${SETUP_INSTALL_DIR}/config/default/cert.pem" "${SETUP_INSTALL_DIR}/config/cert.pem"
        cp -f "${SETUP_INSTALL_DIR}/config/default/content.xml" "${SETUP_INSTALL_DIR}/config/content.xml"
        cp -f "${SETUP_INSTALL_DIR}/config/default/imservices.dat" "${SETUP_INSTALL_DIR}/config/imservices.dat"
        cp -f "${SETUP_INSTALL_DIR}/config/default/servicebind.dat" "${SETUP_INSTALL_DIR}/config/servicebind.dat"
        cp -f "${SETUP_INSTALL_DIR}/config/default/webserver.dat" "${SETUP_INSTALL_DIR}/config/webserver.dat"

        # antispam
        cp -f "${SETUP_INSTALL_DIR}/spam/default/db/antispam.db" "${SETUP_INSTALL_DIR}/spam/antispam.db"
        cp -f "${SETUP_INSTALL_DIR}/spam/default/rules/local.cf" "${SETUP_INSTALL_DIR}/spam/rules/local.cf"
    fi

    # In case of upgrading, check for missing configuration files
    # this is done in function check_settings_files()
}

configure_installation()
{
	DATE=`date`

    # Print IceWarp Server configuration file
    DIR_CONFIG=`dirname ${SETUP_SERVER_CONF}`
    mkdir -p "${DIR_CONFIG}"
    if [ $? -ne 0 ]; then
        bad "Can not create directory: ${DIR_CONFIG}"
        exit 1
    fi

    touch "$SETUP_SERVER_CONF"
    if [ $? -ne 0 ]; then
        bad "Can not create file: ${SETUP_SERVER_CONF}"
        exit 1
    fi

	echo "# IceWarp Server configuration file
# this file was generated by IceWarp installer on ${DATE}

# Server installation directory
# IWS_INSTALL_DIR=/path/to/server
IWS_INSTALL_DIR=\"${SETUP_INSTALL_DIR}\"

# User to run server as
# IWS_PROCESS_USER=username
IWS_PROCESS_USER=\"${SETUP_INSTALL_USER}\"

# IceWarp Server version, used for upgrade process
# IWS_VERSION=number
IWS_VERSION=\"${SETUP_NEWVERSION}\"

# Previous version number, for information
IWS_OLDVERSION=\"${SETUP_OLDVERSION}\"

" > "$SETUP_SERVER_CONF"

# Print settings to php.ini
echo "
; xcache must be loaded as zend_extension and before Zend optimizer
; otherwise it can cause php crash
zend_extension = \"${SETUP_INSTALL_DIR}/php/ext/xcache.so\"
" >> "${SETUP_INSTALL_DIR}/php/php.ini"

echo "[Zend]
zend_extension_manager.optimizer = \"${SETUP_INSTALL_DIR}/php/ext/Optimizer-3.3.3\"
zend_extension_manager.optimizer_ts = \"${SETUP_INSTALL_DIR}/php/ext/Optimizer_TS-3.3.3\"
zend_optimizer.version = 3.3.3
zend_extension = \"${SETUP_INSTALL_DIR}/php/ext/ZendExtensionManager.so\"
zend_extension_ts = \"${SETUP_INSTALL_DIR}/php/ext/ZendExtensionManager_TS.so\"
" >> "${SETUP_INSTALL_DIR}/php/php.ini"

echo "[IceWarp]
icewarp_sharedlib_path = \"${SETUP_INSTALL_DIR}/html/_shared/\"
extension_dir = \"${SETUP_INSTALL_DIR}/php/ext/\"
error_log = \"${SETUP_INSTALL_DIR}/logs/phperror.log\"
open_basedir_ignore = \"${SETUP_INSTALL_DIR}/\"
upload_tmp_dir = \"${SETUP_INSTALL_DIR}/php/tmp/\"
session.save_path = \"${SETUP_INSTALL_DIR}/php/tmp/\"
sendmail_path = \"${SETUP_INSTALL_DIR}/sendmail\"
" >> "${SETUP_INSTALL_DIR}/php/php.ini"

# Append php.user.ini
if [ -f "${SETUP_INSTALL_DIR}/php/php.user.ini" ]; then
  echo -e "; Content of php.user.ini" >> "${SETUP_INSTALL_DIR}/php/php.ini"
  cat "${SETUP_INSTALL_DIR}/php/php.user.ini" >> "${SETUP_INSTALL_DIR}/php/php.ini"
  echo "" >> "${SETUP_INSTALL_DIR}/php/php.ini"
fi
}

# Function will detect settings path for 
detect_settings_paths()
{
    SETTINGS_DIR_CALENDAR=$("${SETUP_INSTALL_DIR}/tool.sh" display system C_CalendarPath)
    SETTINGS_DIR_CALENDAR=${SETTINGS_DIR_CALENDAR##"C_CalendarPath: "}
    if [ "x${SETTINGS_DIR_CALENDAR}" == "x" ]; then
        SETTINGS_DIR_CALENDAR="${SETUP_INSTALL_DIR}/calendar"
    fi

    SETTINGS_DIR_CONFIG=$("${SETUP_INSTALL_DIR}/tool.sh" display system C_ConfigPath)
    SETTINGS_DIR_CONFIG=${SETTINGS_DIR_CONFIG##"C_ConfigPath: "}
    if [ "x${SETTINGS_DIR_CONFIG}" == "x" ]; then
        SETTINGS_DIR_CONFIG="${SETUP_INSTALL_DIR}/config"
    fi

    SETTINGS_DIR_SPAM=$("${SETUP_INSTALL_DIR}/tool.sh" display system C_SpamPath)
    SETTINGS_DIR_SPAM=${SETTINGS_DIR_SPAM##"C_SpamPath: "}
    if [ "x${SETTINGS_DIR_SPAM}" == "x" ]; then
        SETTINGS_DIR_SPAM="${SETUP_INSTALL_DIR}/spam"
    fi
}

# Helper function for check_settings_files()
# Parameters:
#  $1 - source file name, copy from
#  $2 - destination file name to check and copy to
# Returns:
#  None
check_settings_file()
{
    FILE_NAME_SRC=$1
    FILE_NAME_DST=$2
    if [ ! -f "$FILE_NAME_DST" ]; then
        good "Configuration file ${FILE_NAME_DST} does not exist, copying from default."
        cp -f "$FILE_NAME_SRC" "$FILE_NAME_DST"
        if [ $? -ne 0 ]; then
            bad "Can not copy ${FILE_NAME_SRC} to ${FILE_NAME_DST}"
            exit 1
        fi
    fi
}

# Function upgrades settings files from older versions to newest one
# detect_settings_paths() must be called before this function
# Paramters:
#  none
# Returns:
#  none
check_settings_files()
{
    # this function is needed only for upgrading
    if [ "$UPGRADE" == "0" ]; then
        return 0
    fi

    # extract calendar, config and spam directories into temporary folder
    DIR_TEMP=$(mktemp -d)

    tar -xf "${WORKDIR}/${IMAGENAME}" -C "$DIR_TEMP" calendar
    if [ $? -ne 0 ]; then
        bad "Can not extract calendar files from image"
        exit 1
    fi
    tar -xf "${WORKDIR}/${IMAGENAME}" -C "$DIR_TEMP" config
    if [ $? -ne 0 ]; then
        bad "Can not extract config files from image"
        exit 1
    fi
    tar -xf "${WORKDIR}/${IMAGENAME}" -C "$DIR_TEMP" spam
    if [ $? -ne 0 ]; then
        bad "Can not extract spam files from image"
        exit 1
    fi

    # copy files from temp to destination
    cp -fr "${DIR_TEMP}/calendar/"* "$SETTINGS_DIR_CALENDAR"
    if [ $? -ne 0 ]; then
        bad "Can not copy calendar files"
        exit 1
    fi
    cp -fr "${DIR_TEMP}/config/"* "$SETTINGS_DIR_CONFIG"
    if [ $? -ne 0 ]; then
        bad "Can not copy config files"
        exit 1
    fi
    cp -fr "${DIR_TEMP}/spam/"* "$SETTINGS_DIR_SPAM"
    if [ $? -ne 0 ]; then
        bad "Can not copy spam files"
        exit 1
    fi

    # remove temporary directory
    rm -rf "$DIR_TEMP"

    # test if settings files exists, copy one from default

    # Files in calendar directory

    # calendar/calendar.dat
    FNAME_SRC="${SETTINGS_DIR_CALENDAR}/default/calendar.dat"
    FNAME_DST="${SETTINGS_DIR_CALENDAR}/calendar.dat"
    check_settings_file "$FNAME_SRC" "$FNAME_DST"
    # calendar/groupware.db
    FNAME_SRC="${SETTINGS_DIR_CALENDAR}/default/db/groupware.db"
    FNAME_DST="${SETTINGS_DIR_CALENDAR}/groupware.db"
    check_settings_file "$FNAME_SRC" "$FNAME_DST"

    # Files in config directory

    # config/cert.pem
    FNAME_SRC="${SETTINGS_DIR_CONFIG}/default/cert.pem"
    FNAME_DST="${SETTINGS_DIR_CONFIG}/cert.pem"
    check_settings_file "$FNAME_SRC" "$FNAME_DST"
    # config/content.xml
    FNAME_SRC="${SETTINGS_DIR_CONFIG}/default/content.xml"
    FNAME_DST="${SETTINGS_DIR_CONFIG}/content.xml"
    check_settings_file "$FNAME_SRC" "$FNAME_DST"
    # config/imservices.dat
    FNAME_SRC="${SETTINGS_DIR_CONFIG}/default/imservices.dat"
    FNAME_DST="${SETTINGS_DIR_CONFIG}/imservices.dat"
    check_settings_file "$FNAME_SRC" "$FNAME_DST"
    # config/servicebind.dat
    FNAME_SRC="${SETTINGS_DIR_CONFIG}/default/servicebind.dat"
    FNAME_DST="${SETTINGS_DIR_CONFIG}/servicebind.dat"
    check_settings_file "$FNAME_SRC" "$FNAME_DST"
    # config/webserver.dat
    FNAME_SRC="${SETTINGS_DIR_CONFIG}/default/webserver.dat"
    FNAME_DST="${SETTINGS_DIR_CONFIG}/webserver.dat"
    check_settings_file "$FNAME_SRC" "$FNAME_DST"

    # Files in spam directory

    # spam/antispam.db
    FNAME_SRC="${SETTINGS_DIR_SPAM}/default/db/antispam.db"
    FNAME_DST="${SETTINGS_DIR_SPAM}/antispam.db"
    check_settings_file "$FNAME_SRC" "$FNAME_DST"
    # spam/rules/local.cf
    FNAME_SRC="${SETTINGS_DIR_SPAM}/default/rules/local.cf"
    FNAME_DST="${SETTINGS_DIR_SPAM}/rules/local.cf"
    check_settings_file "$FNAME_SRC" "$FNAME_DST"

    return 0
}

# Function installs service to system
# Paramters:
#  none
# Returns:
#  none
install_service()
{
    # add/remove merakd as system service
    SERVICE_INSTALLED="0"
    SERVICE_STARTED="0"
    if [ "x${OS_DIST}" != "xRHEL" ]; then
        return 0
    fi

    # check if merak service is installed
    good "Checking for previous 'merak' service installation ..."
    service merak status 2> /dev/null
    if [ $? -eq 0 ]; then
        good "Service 'merak' was found installed in system"
        good "which is now obsolete and renamed to 'icewarp' service."
        ask_with_confirmation "Delete service 'merak'?" "Y" "n"
        if [ $? -eq 1 ]; then
            good "Removing 'merak' system service ..."
            chkconfig --del merak > /dev/null
            rm -f "/etc/init.d/merak"
        fi
    fi

    # check if system service is installed
    good "Checking if IceWarp Server is added as system service ..."
    good "Note: System service can be reinstalled"
    good "      by removing already installed service."
    good ""
    service "$SETUP_INSTALL_SERVICE_NAME" status > /dev/null
    if [ $? -eq 0 ]; then
        SERVICE_INSTALLED="1"
        good "IceWarp Server is already added as system service."
        ask_with_confirmation "Delete service?" "Y" "n"
        if [ $? -eq 1 ]; then
            good "Removing IceWarp Server system service ..."
            chkconfig --del "$SETUP_INSTALL_SERVICE_NAME" > /dev/null
            rm -f "/etc/init.d/${SETUP_INSTALL_SERVICE_NAME}"
            SERVICE_INSTALLED="0"
        fi
    fi

    # ask and add icewarpd as service
    if [ "$SERVICE_INSTALLED" == "0" ]; then
        ask_with_confirmation "Do you want to add IceWarp Server as a system service?" "Y" "n"
        if [ $? -eq 1 ]; then
            good "Adding IceWarp Server as system service"
            SERVICE_FILE="/etc/init.d/${SETUP_INSTALL_SERVICE_NAME}"
            # create new service script
            sed "s|%%IWS_INSTALL_DIR_VALUE%%|${SETUP_INSTALL_DIR}|" "${SETUP_INSTALL_DIR}/scripts/setup/${SETUP_INSTALL_SERVICE_NAME}.init" > "$SERVICE_FILE"
            if [ $? -ne 0 ]; then
                bad "Can not create service file."
                exit 1
            fi
            chmod a+x "$SERVICE_FILE" > /dev/null
            if [ $? -ne 0 ]; then
                bad "Cannot change permissions on file: ${SERVICE_FILE}"
                exit 1
            fi
            SERVICE_INSTALLED="1"
        else
            good "IceWarp Server will not be added as system service."
        fi
    fi

    # ask to add or remove service from runlevel
    if [ "$SERVICE_INSTALLED" == "1" ]; then
        chkconfig --del "$SETUP_INSTALL_SERVICE_NAME" > /dev/null
        ask_with_confirmation "Do you want to start IceWarp Server on system startup?" "Y" "n"
        if [ $? -eq 1 ]; then
            good "Making service starting on system startup ..."
            chkconfig --add "$SETUP_INSTALL_SERVICE_NAME" > /dev/null
            if [ $? -ne 0 ]; then
                bad "Error setting runlevel."
                exit 1
            fi
        else
            good "IceWarp Server will not start automatically on system startup."
        fi
    fi
}

# Function upgrades server data and tables from previous version to latest
# Parameters:
#  None
# Returns:
#  None
upgrade_server_data()
{
    # run upgrade tool if upgrading
    if [ "$UPGRADE" != "1" ]; then
        return 0
    fi

    good ""
    good "It is possible now to upgrade data to the current version."
    good "To do this, IceWarp Server must be started."
    good "Note: You can run upgrade process later by command"
    good "      in ${SETUP_INSTALL_DIR}: ./tool.sh upgrade _old_version_"
    good ""
    ask_with_confirmation "Start services and perform the data upgrade now?" "Y" "n"
    if [ $? -eq 1 ]; then
        good "Starting IceWarp Server ..."
        "${SETUP_INSTALL_DIR}/icewarpd.sh" --start > /dev/null
        if [ $? -ne 0 ]; then
            bad "Cannot start IceWarp Server"
            exit 1
        else
            MERAK_STARTED="1"
        fi
        good "Upgrading data ..."
        sleep 10
        "${SETUP_INSTALL_DIR}/tool.sh" upgrade "$SETUP_OLDVERSION" > /dev/null
        if [ $? -eq 0 ]; then
            sleep 10
            good "Data upgrade completed successfully"
        else
            bad "Data upgrade problem!"
        fi
    fi
}

create_uninstall_info()
{
    # create list of all files in tar
    tar -tf "${WORKDIR}/${IMAGENAME}" > "${SETUP_INSTALL_DIR}/scripts/setup/content.txt" > /dev/null

    # split tar content to directories and files
    INSTALLED_DIRS=$(cat "${SETUP_INSTALL_DIR}/scripts/setup/content.txt" | grep ".*\/$")
    INSTALLED_FILES=$(cat "${SETUP_INSTALL_DIR}/scripts/setup/content.txt" | grep ".*[^\/]$")

    # create file with installed directories
    INSTALLED_DIRS_FILE="${SETUP_INSTALL_DIR}/scripts/setup/installed_dirs.txt"
    rm -f "$INSTALLED_DIRS_FILE"
    touch "$INSTALLED_DIRS_FILE"
    for LINE in $INSTALLED_DIRS; do
        echo $LINE >> "$INSTALLED_DIRS_FILE"
    done

    # create file with installed files
    # do not count configuration files which comes with installation
    INSTALLED_FILES_FILE="${SETUP_INSTALL_DIR}/scripts/setup/installed_files.txt"
    rm -f "$INSTALLED_FILES_FILE"
    touch "$INSTALLED_FILES_FILE"
    for LINE in $INSTALLED_FILES; do
        if  [ "x$LINE" != "xconfig/cert.pem" ] &&
            [ "x$LINE" != "xconfig/webserver.dat" ] &&
            [ "x$LINE" != "xspam/antispam.db" ] &&
            [ "x$LINE" != "xspam/rules/local.cf" ] &&
            [ "x$LINE" != "xcalendar/groupware.db" ]
        then
            echo $LINE >> "$INSTALLED_FILES_FILE"
        fi
    done

    # add installed_dirs.txt file
    echo "scripts/setup/installed_dirs.txt" >> "$INSTALLED_DIRS_FILE"
    # add installed_files.txt file
    echo "scripts/setup/installed_files.txt" >> "$INSTALLED_FILES_FILE"

    # remove content file
    rm -f "${SETUP_INSTALL_DIR}/scripts/setup/content.txt"
}

change_user_owner()
{
    # All files are installed OK, change permissions
    good "Changing permissions ..."
    chown -R "${SETUP_INSTALL_USER}:${SETUP_INSTALL_GROUP}" "$SETUP_INSTALL_DIR"
    if [ $? -ne 0 ]; then
        bad "Failed! Check user permissions on ${SETUP_INSTALL_DIR}"
        bad "Setup continues ..."
    fi
}

final_cleanup()
{
    # binaries
    rm -f "${SETUP_INSTALL_DIR}/merakd"
    rm -f "${SETUP_INSTALL_DIR}/startd"
    rm -f "${SETUP_INSTALL_DIR}/stopd"
    rm -f "${SETUP_INSTALL_DIR}/restartd"
    # scripts
    rm -f "${SETUP_INSTALL_DIR}/merakd.sh"
    # sockets
    rm -f "${SETUP_INSTALL_DIR}/var/icewarpd.sock"
    rm -f "${SETUP_INSTALL_DIR}/var/phpsocket"
    # previous configuration
    rm -f "${SETUP_INSTALL_DIR}/config/icewarp.conf"
    rm -f "${SETUP_INSTALL_DIR}/config/merak.conf"
}

run_wizard()
{
    # run wizard if not upgrading
    if [ "$UPGRADE" != "0" ]; then
        return
    fi

    good ""
    ask_with_confirmation "Do you want to run IceWarp Server Wizard now?" "Y" "n"
    if [ $? -ne 1 ]; then
        good "You can run wizard at a later time using command: ${SETUP_INSTALL_DIR}/wizard.sh"
        return
    fi

    if [ "$SERVICE_STARTED" == "0" ]; then
        good "Starting IceWarp Server ..."
        "${SETUP_INSTALL_DIR}/icewarpd.sh" --start > /dev/null
        if [ $? -ne 0 ]; then
            bad "Cannot start IceWarp Server"
            exit 1
        fi
    fi
    "${SETUP_INSTALL_DIR}/wizard.sh"
}

rejoice()
{
    good ""
    if [ "$UPGRADE" != "0" ]; then
        good "IceWarp Server was successfully installed."
    else
        good "IceWarp Server was successfully upgraded."
    fi
    good ""
    good "Installer log is available in ${INSTALL_LOG}"
    good "Installer error log is available in ${INSTALL_ERROR_LOG}"
}

copyright()
{
    good ""
    good "IceWarp Server installer"
    good "(c) 1999-2008 IceWarp Ltd. "
    good ""
}

getscriptparams()
{
    case $1 in
	"-h"|"--help")
	    copyright
	    good ""
	    good "Usage: $0"
	    good ""
	    good "This script provides functionality for installing or upgrading IceWarp Server."
	    good "Default action is to install but if the following conditions are met,"
	    good "only upgrade will be performed:"
	    good "  1. entered destination directory exists"
	    good "  2. file INSTALL_DESTINATION_DIRECTORY/config/merak.conf or"
	    good "     file INSTALL_DESTINATION_DIRECTORY/config/icewarp.conf exists"
	    good ""
	    good "Install mode extracts all files and creates configuration files"
	    good "from default configuration file directories."
	    good ""
	    good "Upgrade mode:"
	    good "  1. replaces all files which are not configuration files"
	    good "     (i.e. files not modified by running server)"
	    good "  2. recreates default configuration files that do not exists:"
	    good "       ./calendar/calendar.dat"
	    good "       ./calendar/groupware.db"
	    good "       ./config/cert.pem"
	    good "       ./config/webserver.dat"
	    good "       ./spam/antispam.dat"
	    good "       ./spam/rules/local.cf"
	    good ""
	    good "[END]"
	;;
	*)
	    accept_license
	    check_server_is_running
	    good "Installation/Upgrade mode entered"
	    good ""
	    #
	    # Part 1.
	    #
	    # extract server files, and create server configuration file
	    # so server can be started. Some settings files can be incorrect
	    # which will be fixed in Part 2. 
	    #
	    detect_install_options
	    ask_questions
	    check_install_options
	    confirm_install_options
	    extract_package
	    configure_installation          # Generate configuration files
	    #
	    # Part 2.
	    #
	    # all files required for server starting are now in place:
	    # image is extracted, server configuration file is generated
	    #
	    detect_settings_paths           # counts with load balanced paths
	    check_settings_files            # checks if setting file is missing, if yes replace with default one
	    install_service
	    upgrade_server_data
	    create_uninstall_info
	    change_user_owner
	    #
	    # Part 3.
	    #
	    # server is fully installed and configured, delete not needed things
	    final_cleanup
	    #
	    # Part 4.
	    #
	    # run user configuration
	    run_wizard
	    rejoice
	;;
    esac
}

# this install script can be executed only by user with root privileges
if [ "x${RUNNING_UID}" != "x0" ]; then
    bad "This install scripts can be executed by root user only."
    exit 1
fi

DATE=$(date)

touch "$INSTALL_LOG"
touch "$INSTALL_ERROR_LOG"

echo "INSTALLATION STARTED ON ${DATE}" > "$INSTALL_LOG"
echo "INSTALLATION STARTED ON ${DATE}" > "$INSTALL_ERROR_LOG"
detect_os_distribution
getscriptparams $* 2>> "$INSTALL_ERROR_LOG" | tee -a "$INSTALL_LOG"
echo "INSTALLATION FINISHED ON ${DATE}" >> "$INSTALL_LOG"
echo "INSTALLATION FINISHED ON ${DATE}" >> "$INSTALL_ERROR_LOG"

exit 0
