// INIT script

/**
 * @brief : Initial class for IceWarp WebMail
 * @date  : 6.3.2006 19:01:49
 * @status: draft
 * @require: storage
 *

  ? check opener (for valid webmail window)
  - parse input vars to GLOBAL cookie_vars and url_vars
  - choose page ($page <- "login" <- cookie_vars <- url_vars)
  - check ID (!isset than page = login)
  - load settings (if doesnt exist in [opener.]dataset)
  - get Lang (depeds on settings)
*/

function cInit(){

	//registr function to global variable, must be here because it will be used
	window.oWM_INIT = this;

	// hide copyright box
	var eCopyright = document.getElementById('wm_copyright');
	if (eCopyright){
		eCopyright.style.display = 'none';
		eCopyright = null;
	}

	// allowed content
	var allowed_cookie = {"sid":'',"frm":''},
		allowed_get    = {"page":'index',"debug":null,"frm":null,"sid":null,"ref":null,"user":null,"pass":null,"mailto":null,"video":null,"lang":null};

	// parse cookies and get vars
	gui._REQUEST_VARS = arrConcat(arrKeySlice(cookieManager.get('wm_cookie') || [], allowed_cookie), arrKeySlice(parseURL(), allowed_get));

	// DEBUG
	//if (gui._REQUEST_VARS['debug']) window.api_debug = new cAPI_debug();

	storage.library('gw_others');
	GWOthers.load(['skins','login_settings','layout_settings','forgot_settings','restrictions','signup_domains','languages','paths']);

	// get Language
	if (GWOthers.getItem('RESTRICTIONS','disable_languages')>0)
		storage.language(GWOthers.getItem('LAYOUT_SETTINGS','language'));
	else
		storage.language(gui._REQUEST_VARS['lang'] || cookieManager.get('LoginLang') || GWOthers.getItem('LAYOUT_SETTINGS','language'));

	// create connection object
	gui._create('connection','obj_connection');

	if (gui._REQUEST_VARS['sid']){
		var sURL,sid = auth.login({"session":gui._REQUEST_VARS['sid']});

		//Add SID into dataset
		if (sid)
			dataSet.add('main',['sid'],sid);
		else{
			if (Is.Object(auth.error)){
				switch (auth.error.id){
					case 'login_account_valid':
						alert(getLang('ERROR::ACCOUNT_DISABLED'));
						break;
					case 'login_invalid':
					case 'session_no_user':
						alert(getLang('ERROR::INVALID_LOGIN'));
						break;
					default:
						inspect2(auth.error);
				}
			}
			else
				alert(getLang('ERROR::INVALID_LOGIN'));

			if ((sURL = GWOthers.getItem('LAYOUT_SETTINGS','logout_url') || gui._REQUEST_VARS['ref']))
				document.location.replace(sURL);
            else
				gui._create('frm_login','frm_login','','',1);
				
			return;
		}

		//Save referrer, WM will be redirected back to this adress after logout
		if (gui._REQUEST_VARS['ref'])
			dataSet.add('main',['referrer_url'],gui._REQUEST_VARS['ref']);
		else
		if (document.referrer && document.referrer != document.location.href)
			dataSet.add('main',['referrer_url'], document.referrer);

		//start login sequence
		this._checkBrowserVersion();
		return;
	}

	if (gui._REQUEST_VARS['mailto']){
		try{
			var sid = '';
			if (gui._REQUEST_VARS['sid'])
				sid = auth.login({"session":gui._REQUEST_VARS['sid']});
			else
			if (gui._REQUEST_VARS['user'] && gui._REQUEST_VARS['pass'])
				sid = auth.login({"username":gui._REQUEST_VARS['user'],"password":gui._REQUEST_VARS['pass']});
			else{
				var data = cookieManager.get('LoginData') || {},
					state = cookieManager.get('LoginState');

				if ((state==2 || state==3) && data.user && data.pass)
					sid = auth.login({"username":data.user,"digest":data.pass});
			}
		}
		catch(r){
			alert(getLang('ERROR::CONNECTION')+(typeof r == 'string'?":\n"+r:''));
		}

		//Login was successful
		if (sid && !auth.error){

			//Hide copyright block
			var celm;
			if((celm = document.getElementById('gui.svn'))){
				celm.style.display = 'none';
				celm = null;
			}

			this._checkBrowserVersion();
			return;
		}
		else{
			if (Is.Object(auth.error)){
				switch (auth.error.id){
					case 'login_account_valid':
						alert(getLang('ERROR::ACCOUNT_DISABLED'));
						break;
					case 'login_invalid':
						alert(getLang('ERROR::INVALID_LOGIN'));
						break;
				}
			}
			else
				alert(getLang('ERROR::INVALID_LOGIN'));

			gui._create('frm_login','frm_login','','',1);
		}
	}
	else
	switch (gui._REQUEST_VARS['frm']){
	// MAIN page
	case 'main':
		auth.login({"username":gui._REQUEST_VARS['user'] || 'admin',"password":gui._REQUEST_VARS['pass'] || 'asd'});
		gui._create('frm_main','frm_main');
		break;

	// LOGIN page
	default:
		gui._create('frm_login','frm_login','','',1);
	}
};

/**
 * Login sequence
 * @note: moved from frm_login (4.7.2007 10:58:11)
 **/
cInit.prototype._checkBrowserVersion = function () {

	switch (currentBrowser()) {

		case 'Mozilla':
		case 'MSIE7':
		case 'Opera':
		case 'Safari':
		case 'Chrome':
			this._continueLogin(); // do nothing, the browser is supported
			break;

		case 'MSIE6':

			storage.library('wm_storage');
			storage.library('gw_others');

			if (GWOthers.getItem('RESTRICTIONS', 'disable_ie6warning')){
				this._continueLogin();
				break;
			}

			//nacist "resources" do datasetu "storage"
			dataSet.add('storage','',WMStorage.get({'resources':['login_data']}));

			//precist data z datasetu "storage"
			var iLastShow = parseInt(GWOthers.getItem('LOGIN_DATA', 'ie_6_warning_show_on'));

			var iNow = parseInt((new Date()).getTime());
			if (!Is.Number(iLastShow) || iLastShow <= iNow-(this._showIEWarningFrequency*86400000)){
				GWOthers.set('LOGIN_DATA',{'ie_6_warning_show_on':iNow},'storage');
				WMStorage.set({'resources':dataSet.get('storage')},'storage');

				var cfm = gui._create('frm_confirm','frm_confirm','','', [this, '_continueLogin'], 'CONFIRMATION::BROWSER_WARNING_TITLE','CONFIRMATION::BROWSER_WARNING_TEXT_IE6');
				    cfm.btn_cancel._onclick = function() {
						cfm._destruct();
						document.location.href = './basic';
					};
			}
			else
				this._continueLogin();

			break;

		default:
			if (window.confirm(getLang('CONFIRMATION::BROWSER_WARNING_TEXT_UNSUPPORTED')))
				this._continueLogin();
			else
				document.location.href = './basic';
				
			break;
	}
};

cInit.prototype._continueLogin = function (b) {

	var me = this;

	if (!gui.preloader)
		gui._create("preloader","obj_loader");

	// [Sync] load of BIG JavaSript
	setTimeout(function(){if (gui.preloader) gui.preloader._value(getLang('PRELOADER::FRAMEWORK'));},0);
	storage.library('javascript');

	var old_skin = GWOthers.getItem('LAYOUT_SETTINGS', 'skin');

	// [Sync] load resources
	setTimeout(function(){if (gui.preloader) gui.preloader._value(getLang('PRELOADER::SETTINGS'));},0);
	GWOthers.load([	'skins',
					'im',
					'sip',
					'mail_settings_default',
					'mail_settings_general',
					'login_settings',
					'layout_settings',
					'homepage_settings',
					'calendar_settings',
					'default_calendar_settings',
					'cookie_settings',
					'default_reminder_settings',
					'event_settings',
				//	'default_folders',
					'spellchecker_languages',
					'signature',
					'restrictions',
					'personalities',
					'read_confirmation',
					'global_settings',
					'paths',
					'streamhost',
					'google']);

	// Clear already downloaded templates
	if (old_skin != GWOthers.getItem('LAYOUT_SETTINGS', 'skin'))
		storage.aStorage.template = {};

	// [Sync] load language
	setTimeout(function(){if (gui.preloader) gui.preloader._value(getLang('PRELOADER::LANGUAGE'));},0);

	if (GWOthers.getItemAccess('LAYOUT_SETTINGS','language') && (GWOthers.getItem('RESTRICTIONS','disable_languages') || 0)<1)
		storage.language(gui._REQUEST_VARS['lang'] || cookieManager.get('LoginLang') || GWOthers.getItem('LAYOUT_SETTINGS','language'));
	else
		storage.language(GWOthers.getItem('LAYOUT_SETTINGS','language'));

	// get accounts
	var aAccounts = WMAccounts.list();
	dataSet.add('accounts','',aAccounts);

	// refresh main account & continue preloading...
	this.__refreshed = false;
	WMFolders.list({'aid':sPrimaryAccount},'folders',[sPrimaryAccount],[me,'_updatePreloader']);

	/*
	for (var sAccId in aAccounts)
		if (aAccounts[sAccId] && aAccounts[sAccId]['PRIMARY']){
			// call the refresh via timeout so that the preloader is shown properly
			setTimeout(
			function(){

				if (gui.preloader)
					gui.preloader._value(getLang('PRELOADER::INDEXING'));

				WMAccounts.refresh({'aid':sAccId},'folders',[sAccId],[me,'_updatePreloader']);
			},0);
			break;
		}
	*/
};

cInit.prototype._updatePreloader = function (sError) {

	if (!dataSet.get('folders',[sPrimaryAccount])){
		this.__refreshed = true;
		setTimeout(function(){if (gui.preloader) gui.preloader._value(getLang('PRELOADER::INDEXING'));},0);
		WMAccounts.refresh({'aid':sPrimaryAccount},'folders',[sPrimaryAccount],[this,'_updatePreloader']);
		return;
	}

	// works only with WMAccounts
	if (typeof sError == 'string'){
		var state = cookieManager.get('LoginState');
		if (state === '3')
			cookieManager.set('LoginState','2',30);

		alert(sError);

		window.location.href = window.location.href;
		return;
	}

	// call the refresh via timeout so that the preloader is updated properly
	var me = this;
	setTimeout(function(){
		if (gui.preloader)
			gui.preloader._value(getLang('PRELOADER::STARTUP'));
			
		me._startMain();
	},0);
}

cInit.prototype._startMain = function () {
	// [Async] Load style.css (marged css files)
	storage.css('style');
	storage.template('templates');
	storage.preloadObj();

	if (gui.preloader)
		gui.preloader._destruct();

	gui._create("frm_main","frm_main","","",true);

	if (gui._REQUEST_VARS['mailto']){

		var head = {};
		if (gui._REQUEST_VARS['mailto'].indexOf('?')>-1){
			head = parseURL(gui._REQUEST_VARS['mailto']);
			head.to = unescape(gui._REQUEST_VARS['mailto'].substr(0,gui._REQUEST_VARS['mailto'].indexOf('?')));
		}
		else
			head.to = unescape(gui._REQUEST_VARS['mailto']);

		NewMessage.compose(head);
	}
	
/*
	if (!this.__refreshed)
		WMAccounts.refresh({'aid':sPrimaryAccount},'folders',[sPrimaryAccount]);
*/
};
///////////////////////////
function initgui(){
	new cInit();
};