#!/bin/bash
#
# IceWarp Server uninstall script
# Copyright (c) 2008-2010 IceWarp Ltd. All rights reserved.
#
# http://www.icewarp.com
#
# file: uninstall.sh - uninstall script
#

# init working dir
pushd "$(pwd)" &> /dev/null
WORK_DIR=$(dirname "$0")
cd "$WORK_DIR" &> /dev/null
if [ $? -ne 0 ]; then
    echo "Can not change directory to ${WORK_DIR}"
    exit 1
fi

# include functions
if [ -z "$FUNCTIONS_INCLUDED" ]; then
    source "./inc/functions.sh"
fi

# start uninstall
uninstall_cancelled_quit()
{
    if [ -n "$1" ]; then
        echo "$1"
    fi
    echo "Uninstall cancelled."
    exit 1;
}

# TODO add information print that server must be stopped

ask_with_confirmation "UNINSTALL IceWarp Server FROM ${IWS_INSTALL_DIR}?" "N" "y"
if [ $? -ne 2 ]; then
    uninstall_cancelled_quit
fi

echo "Uninstalling IceWarp Server..."

# load directories and files list
if [ -f "${IWS_INSTALL_DIR}/scripts/setup/installed_dirs.txt" ]; then
    INSTALLED_DIRS=$(cat "${IWS_INSTALL_DIR}/scripts/setup/installed_dirs.txt")
else
    uninstall_cancelled_quit "Can not find istalled directories list file."
fi
if [ -f "${IWS_INSTALL_DIR}/scripts/setup/installed_files.txt" ]; then
    INSTALLED_FILES=$(cat "${IWS_INSTALL_DIR}/scripts/setup/installed_files.txt")
else
    uninstall_cancelled_quit "Can not find istalled files list file."
fi

# change to install directory
cd "$IWS_INSTALL_DIR" &> /dev/null
if [ $? -ne 0 ]; then
    uninstall_cancelled_quit "Can not change directory to ${IWS_INSTALL_DIR}"
fi

# remove all installed files
for LINE in $INSTALLED_FILES
do
    rm -f "${IWS_INSTALL_DIR}/${LINE}"
done

# remove all installed directories
for LINE in $INSTALLED_DIRS
do
    rmdir -p "${IWS_INSTALL_DIR}/${LINE}" &> /dev/null
done

# remove top directory
cd "${IWS_INSTALL_DIR}/.."
rmdir "$IWS_INSTALL_DIR"
if [ -d "$IWS_INSTALL_DIR" ]; then
    echo "Directory ${IWS_INSTALL_DIR} was not removed."
fi

# unregister service for RHEL and ROOT
UID=$(id -u)
OS_DISTRIBUTION="UNKNOWN"
if [ -f "/etc/redhat-release" ]; then
    OS_DISTRIBUTION="RHEL"
fi
if [ "$OS_DISTRIBUTION" == "RHEL" ]; then
    echo "Trying to remove 'icewarp' service from system ..."
    chkconfig --del icewarp &> /dev/null
    rm -f "/etc/init.d/icewarp" &> /dev/null
fi

# leave configuration file in /etc/icewarp/icewarp.conf
echo "WARNING: Not deleting configuration file: /etc/icewarp/icewarp.conf"

echo
echo "Uninstall done."

# restore working dir
popd &> /dev/null
exit 0
