#!/bin/bash
#
# IceWarp Server
# Copyright (c) 2008-2010 IceWarp Ltd. All rights reserved.
#
# http://www.icewarp.com
#
# file: createtables.sh - wizard menu script
#

if [ "x${IWS_INSTALL_DIR}" == "x" ]; then
    echo "This script needs environment variable IWS_INSTALL_DIR"
    exit 1
fi

if [ "x${FUNCTIONS_INCLUDED}" == "x" ]; then
    source "${IWS_INSTALL_DIR}/scripts/inc/functions.sh"
fi

createtables()
{
    SERVICETYPE="$1"
    SERVICENAME="$2"
    good ""
    hilite "Set up connection for $SERVICENAME:"
    good ""

    DBNAME="icewarp_server"
    DBUSER="icewarp_server"
    DBHOST=$(grep "^socket" /etc/mysql/my.cnf /etc/my.cnf ~/.my.cnf 2>/dev/null | cut -d'=' -f2 | cut -d' ' -f2 | tail -n 1)

    getparam "Database name [${DBNAME}]:"
    if [ "x${PARAM}" != "x" ]; then
        DBNAME=$PARAM
    fi

    getparam "Database username [${DBUSER}]:"
    if [ "x${PARAM}" != "x" ]; then
        DBUSER=$PARAM
    fi

    getpassword "Database password:"
    DBUSERPW=$SPARAM

    getparam "Database server (socket path or host or leave blank) [${DBHOST}]:"
    if [ "x${PARAM}" != "x" ]; then
        DBHOST=$PARAM
    fi

    ask_with_confirmation "OK?" "Y" "n"
    if [ $? -ne 1 ]; then
        return 0
    fi

    DBNEWUSER=$DBUSER
    DBNEWNAME=$DBNAME
    DBNEWUSERPW=$DBUSERPW
    DBNEWHOST=$DBHOST

    mysqlcall
    if [ $? -ne 0 ]; then

	ask_with_confirmation "MySQL login failed using supplied credentials, create new database for this user?" "Y" "n"
	if [ $? -ne 1 ]; then
	    return 0
	fi

	DBUSER="root"
	DBUSERPW=""
	unset DBNAME

	getparam "MySQL administrator username [${DBUSER}]:"
	if [ "x${PARAM}" != "x" ]; then
	    DBUSER=$PARAM
	fi

	getpassword "MySQL administrator password:"
	DBUSERPW=$SPARAM

	getparam "MySQL server (socket path or host or leave blank) [${DBHOST}]:"
	if [ "x${PARAM}" != "x" ]; then
	    DBHOST=$PARAM
	fi

	good ""
	ask_with_confirmation "Generate a secure password for the new user (uuencode needed)?" "Y" "n"
	if [ $? -eq 1 ]; then
	    DBNEWUSERPW=$(head -c 10 /dev/random | uuencode -m - | cut -d'=' -f'1' | tail -n 2 | head -n 1 2>/dev/null)
	    good "Password is: ${DBNEWUSERPW}"
	fi

	getconfirm "${GOOD}OK${HILITE}?${NORMAL}" || return 0
	if [ $? -ne 1 ]; then
	    return 0
	fi

	good ""

	SQLQUERY="CREATE DATABASE ${DBNEWNAME}"
	mysqlcall
	if [ $? -eq 0 ]; then
	    good "Database created"
	else
	    bad "Database creation problem!"
	fi

	SQLQUERY="GRANT ALL ON ${DBNEWNAME}.* TO ${DBNEWUSER}@'localhost' IDENTIFIED BY '${DBNEWUSERPW}'"
	mysqlcall
	if [ $? -eq 0 ]; then
	    good "Privileges granted (${DBNEWUSER}@'localhost')"
	else
	    bad "Unable to grant privileges!"
	fi

	SQLQUERY="GRANT ALL ON ${DBNEWNAME}.* TO ${DBNEWUSER}@'%' IDENTIFIED BY '${DBNEWUSERPW}'"
	mysqlcall
	if [ $? -eq 0 ]; then
	     good "Privileges granted (${DBNEWUSER}@'%')"
	else
	    bad "Unable to grant privileges!"
	fi

	SQLQUERY="GRANT ALL ON ${DBNEWNAME}.* TO ${DBNEWUSER}@'127.0.0.1' IDENTIFIED BY '${DBNEWUSERPW}'"
	mysqlcall
	if [ $? -eq 0 ]; then
	    good "Privileges granted (${DBNEWUSER}@'127.0.0.1')"
	else
	    bad "Unable to grant privileges!"
	fi

	SQLQUERY="FLUSH PRIVILEGES"
	mysqlcall

	ask_with_confirmation "Continue with table creation?" "Y" "n"
	if [ $? -ne 1 ]; then
	    exit 0
	fi
    fi

    run_cmd_log "${IWS_INSTALL_DIR}/tool.sh" "CREATE" "TABLES" "${SERVICETYPE}" "${DBNEWNAME};${DBNEWUSER};${DBNEWUSERPW};${DBNEWHOST};3;2" > /dev/null
    if [ $? -eq 0 ]; then
	good "Tables created" 
    else
        bad "Tables already exist or save problem" 
    fi
    return 0
}

case `basename "$0"` in
    createtables-accounts)
	createtables "0" "Accounts storage"
	if [ $? -ne 0 ]; then
	    storageswitch-convert-DB
	fi
	echo "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  " $?
	;;
    createtables-groupware)
	if [ $? -ne 0 ]; then
	    verybad "GroupWare daemon must be running!"
	else
	    createtables "2" "GroupWare storage"
	fi
	;;
    createtables-challengeresponse)
	createtables "3" "Challenge-response storage"
	;;
    createmysqldb)
	createdbmysql $*
	;;
esac

exit 0
